﻿using Fleck;
using Newtonsoft.Json;
using NLog;
using System;
using System.Collections.Generic;

/*
 *         Progetto: Progetto "Pagamenti MSMQ"
 * 
 *           Autore: Roberto Fuligni
 *  Ultima modifica: 24/01/2016
 *  
 *      Descrizione: Routine di gestione della comunicazione lato server basata su web socket.
 *                   
 */

namespace PagamentiMSMQ
{

    public static class GestoreWebSocket
    {
        private class RecordInvio
        {
            public string DataInvio { get; set; }
            public InfoPagamentoEsteso[] Info { get; set; }
        }

        private static Logger logger = LogManager.GetCurrentClassLogger();
        private static WebSocketServer ws = null;
        private static HashSet<IWebSocketConnection> connessioni = new HashSet<IWebSocketConnection>();
        private static Object mutex = new Object();
        private static bool avviato = false;
        private static InfoPagamentoEsteso[] cacheRecord = null;

        public static void Avvia()
        {
            lock (mutex)
            {
                if (avviato) return;

                avviato = true;
            }

            UtilityDB.Instance.notificaNuovaRevisione += InviaRisposte;
            logger.Trace("WebSocketServer avviato.");
        }

        public static void Arresta()
        {
            lock (mutex)
            {
                foreach (var cn in connessioni)
                {
                    var socket = cn;
                    socket.Close();
                }

                connessioni.Clear();
            }
            logger.Trace("WebSocketServer avviato.");
        }

        static GestoreWebSocket()
        {
            lock (mutex)
            {
                // Web socket attivo sulla porta 1800/tcp di tutte le interfacce di rete

                ws = new WebSocketServer("ws://0.0.0.0:1800");

                ws.Start(socket =>
                {
                    socket.OnOpen = () => GestisciApertura(socket);
                    socket.OnClose = () => GestisciChiusura(socket);
                    socket.OnMessage = message => GestisciMessaggio(socket, message);
                });
            }

            logger.Trace("Nuovo WebsocketServer allocato.");

        }

        private static void GestisciMessaggio(IWebSocketConnection sck, string messaggio)
        {
        }

        private static void GestisciApertura(IWebSocketConnection sck)
        {
            lock (mutex)
            {
                connessioni.Add(sck);
                InviaRisposta(sck);
                logger.Trace("Apertura di un nuovo collegamento a WebsocketServer.");
                
            }
        }

        private static void GestisciChiusura(IWebSocketConnection sck)
        {
            lock (mutex)
            {
                connessioni.Remove(sck);
                logger.Trace("Chiusura di un collegamento a WebsocketServer.");
                
            }
        }

        private static void InviaRisposta(IWebSocketConnection socket)
        {
            lock (mutex)
            {
                if (UtilityDB.Instance.DbModificato || cacheRecord == null) {
                    cacheRecord = UtilityDB.Instance.LeggiPagamentiRecenti();
                    UtilityDB.Instance.DbModificato = false;
                }

                var rec = new RecordInvio()
                {
                    DataInvio = DateTime.Now.ToString("dd/MM/yyyy HH:mm:ss"),
                    Info = cacheRecord
                };

                string buffer = JsonConvert.SerializeObject(rec);

                socket.Send(buffer);
            }
        }

        private static void InviaRisposte(object sender, EventArgs e)
        {
            lock (mutex)
            {
                foreach (var cn in connessioni)
                    InviaRisposta(cn);
            }
        }
    }
}
