﻿using System;
using System.Linq;
using System.Messaging;
using System.ServiceProcess;
using NLog;

/*
 *         Progetto: Progetto "Pagamenti MSMQ"
 * 
 *           Autore: Roberto Fuligni
 *  Ultima modifica: 24/01/2016
 *  
 *      Descrizione: Servizio Windows per la ricezione e l'elaborazione asincrona 
 *                   dei messaggi memorizzati nella coda.
 *                   
 */

namespace PagamentiMSMQ
{
    public partial class ServizioPagamento : ServiceBase
    {
        private static Logger logger = LogManager.GetCurrentClassLogger();
        public ServizioPagamento()
        {
            InitializeComponent();
            
        }

        protected override void OnStart(string[] args)
        {
            try
            {
                logger.Info("Servizio avviato.");
                GestoreWebSocket.Avvia();
                coda.BeginReceive();

            }
            catch (Exception e) {
                logger.Error(e);
            }
        }

        protected override void OnStop()
        {
            GestoreWebSocket.Arresta();
            logger.Info("Servizio arrestato.");

        }

        private void coda_ReceiveCompleted(object sender, System.Messaging.ReceiveCompletedEventArgs e)
        {
            try
            {
                string descrErroreInfo = null;
                InfoPagamento inf = null;

                logger.Trace("Ricevuto messaggio (id{0})", e.Message.Id);

                try
                {
                   inf = (InfoPagamento)e.Message.Body;
                   InfoPagamento.Controllo(inf);

                }
                catch (Exception ex)
                {
                    descrErroreInfo = ex.Message;
                }

                if (descrErroreInfo == null)
                {
                    if (!UtilityDB.Instance.ScriviPagamento(inf))
                        logger.Trace("Problemi durante la scrittura nel database.");
                    else
                    {
                        InfoPagamento[] ris = UtilityDB.Instance.LeggiPagamentiRecenti();
                        if (ris.Count() > 0)
                            logger.Trace("Record più recente: {0}", ris[0]);
                    }
                }

                MessageQueue codaRisposta = e.Message.ResponseQueue;

                if (codaRisposta != null)
                {
                    try
                    {
                        Message risp = new Message();
                        risp.CorrelationId = e.Message.Id;
                        risp.Body = descrErroreInfo != null ? descrErroreInfo : "OK";
                        codaRisposta.Send(risp);
                    }
                    catch (MessageQueueException ex)
                    {
                        logger.Error("Errore durante l'invio della risposta: {0}", ex.Message);
                    }
                }

                coda.BeginReceive();
            }
            catch (Exception ex)
            {
                logger.Error(ex);
            }
        }
    }
}
