﻿using System;
using System.Linq;
using System.Text;

/*
 *         Progetto: Progetto "Pagamenti MSMQ"
 * 
 *           Autore: Roberto Fuligni
 *  Ultima modifica: 24/01/2016
 *  
 *      Descrizione: Contiene la struttura dei seguenti oggetti contenenti i messaggi scambiati dalle applicazioni:
 *      
 *                      InfoPagamento: messaggio inviato alla coda MSMQ del server dei pagamenti.
 *                      InfoPagamentoEsteso: messaggio inviato dal server dei pagamenti al report mediante web socket.
 *                   
 */

namespace PagamentiMSMQ
{
    public class InfoPagamento
    {
        public string NomeVenditore { get; set; }
        public int IdVenditore { get; set; }
        public string NomeAcquirente { get; set; }
        public string NumeroCarta { get; set; }
        public int TipoCarta { get; set; }
        public double Importo { get; set; }

        public static InfoPagamento Test(Random r = null)
        {
            InfoPagamento[] v = new InfoPagamento[] {
                new InfoPagamento() {
                    NomeVenditore = "ABC Costruzioni S.R.L.",
                    IdVenditore = 1,
                    NomeAcquirente = "Mario Rossi",
                    NumeroCarta = "1234-7685-3324-4297",
                    TipoCarta = 1,
                    Importo = 123.50
                },
                new InfoPagamento() {
                    NomeVenditore = "Milani Arredamenti S.N.C.",
                    IdVenditore = 3,
                    NomeAcquirente = "Piero Verdi",
                    NumeroCarta = "4324-6526-8636-24356",
                    TipoCarta = 3,
                    Importo = 260.0
                },
                new InfoPagamento() {
                    NomeVenditore = "S.E.T.I. Program",
                    IdVenditore = 2,
                    NomeAcquirente = "Carla Bianchi",
                    NumeroCarta = "2874-3453-8209-5426",
                    TipoCarta = 2,
                    Importo = 54.65
                }

            };

            return v[r == null ? 0 : (r.Next() % v.Count())];

        }

        public override string ToString()
        {
            var sb = new StringBuilder();
            sb.Append(String.Format("Venditore: {0} (id: {1})\n", NomeVenditore, IdVenditore));
            sb.Append(String.Format("Acquirente: {0} \n", NomeAcquirente));
            sb.Append(String.Format("Carta di credito: {0} (tipo: {1})\n", NumeroCarta, TipoCarta));
            sb.Append(String.Format("Importo: {0:N2} Euro\n", Importo));

            return sb.ToString();

        }

        public static void Controllo(InfoPagamento info)
        {
            if (info.NomeVenditore == null || info.NomeVenditore.Trim().Length == 0)
                throw new Exception("Il campo 'NomeVenditore' è obbligatorio.");

            if (info.IdVenditore <= 0)
                throw new Exception("Il campo 'IdVenditore' è obbligatorio e deve essere positivo.");

            if (info.NomeAcquirente == null || info.NomeAcquirente.Trim().Length == 0)
                throw new Exception("Il campo 'NomeAcquirente' è obbligatorio.");

            if (info.NumeroCarta == null || info.NumeroCarta.Trim().Length == 0)
                throw new Exception("Il campo 'NumeroCarta' è obbligatorio.");

            if (info.TipoCarta < 1 || info.TipoCarta > 3)
                throw new Exception("Il campo 'TipoCarta' è obbligatorio e deve essere compreso tra 1 e 3.");

            if (info.IdVenditore <= 0)
                throw new Exception("Il campo 'Importo' è obbligatorio e deve essere positivo.");
        }
    }

    public class InfoPagamentoEsteso : InfoPagamento
    {
        public int Id { get; set; }
        public string Data { get; set; }
        public string ImportoValuta { get; set; }

        public override string ToString()
        {
            var sb = new StringBuilder();

            sb.Append(String.Format("Id {0} del {1}\n", Id, Data));
            sb.Append(base.ToString());

            return sb.ToString();
        }
    }
}
